const arcPbDeployer = require('./../index');
const deploy = require('./../util/deploy');
const logger = require('./../util/logger');

jest.mock('fs');
jest.mock('request');
jest.mock('zip-dir');
jest.useFakeTimers();

const postDeploy = jest.fn();

describe('deploy', function () {
  const successSpy = jest.spyOn(logger, 'deploySuccess');
  const errorSpy = jest.spyOn(logger, 'deployError');
  const deployArgs = {
    endpoint: 'foo',
    username: 'bar',
    password: 'baz',
    path: 'foo/bar/baz',
    name: 'dan',
    environment: 'test',
    postDeploy
  };
  const successArgs = {
    response: { statusCode: 200 },
    body: undefined,
    name: 'dan',
    environment: 'test'
  };
  it('sends a valid request to the deployment endpoint', function () {
    deploy(deployArgs);
    expect(successSpy).toHaveBeenCalledTimes(1);
    expect(successSpy).toHaveBeenCalledWith(successArgs);
    expect(deployArgs.postDeploy).toHaveBeenCalledTimes(1);

    deployArgs.postDeploy.mockReset();
    deployArgs.endpoint = '';
    deploy(deployArgs);
    expect(errorSpy).toHaveBeenCalledTimes(1);
    expect(errorSpy).toHaveBeenCalledWith(new Error('URL did not match API endpoint pattern'));
    expect(deployArgs.postDeploy).toBeCalledWith(false, 'dan', 'test');
  });
});

describe('arc-pb-deployer', function () {
  const successSpy = jest.spyOn(logger, 'bundleSuccess');
  const errorSpy = jest.spyOn(logger, 'bundleError');
  const delay = 2000;
  let args;
  beforeEach(function () {
    args = {
      rootDirectory: 'root/directory',
      endpoint: 'endpoint',
      username: 'username',
      password: 'password',
      environment: 'test',
      postDeploy
    };
  });
  it('generates a file ending in \'.zip\' within a \'deployment\' directory', function () {
    arcPbDeployer(args)();
    jest.advanceTimersByTime(delay);
    expect(successSpy).toHaveBeenCalledTimes(1);
  });
  it('should fail if a root directory is not present', function () {
    args.rootDirectory = '';
    arcPbDeployer(args)();
    jest.advanceTimersByTime(delay);
    expect(errorSpy).toHaveBeenCalledTimes(1);
  });
});
