const fs = require('fs');
const Promise = require('bluebird');
const request = Promise.promisify(require('request'));

const logger = require('./logger');

/**
* @desc Posts the .zip file to the deployer API based on the environment.
* @param {string} path - A string describing the location of the generted .zip file.
* @param {string} name - A string containing the name of the generated .zip file.
* @param {string} endpoint - A string containing the API endpoint the .zip file should deploy to.
* @param {string} username - A string containing the API username.
* @param {string} password - A string containing the API password.
* @param {object} slack - An object containing the Slack channel, icon and username.
**/

module.exports = ({
  environment,
  path,
  name,
  endpoint,
  username,
  password,
  postDeploy
}) => {
  logger.deployInitialize(path, environment);
  const options = {
    url: `https://${username}:${password}@${endpoint}/deployments/pagebuilder/bundles`,
    formData: {
      name,
      bundle: fs.createReadStream(path)
    }
  };

  request.post(options, (error, response, body) => {
    if (error || response.statusCode !== 200) {
      logger.deployError(error);
      if (postDeploy) {
        postDeploy(false, name, environment);
      }
    } else {
      logger.deploySuccess({ response, body, name, environment });
      if (postDeploy) {
        postDeploy(true, name, environment);
      }
    }
  });
};
